@@ -13,10 +13,12 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <https://www.gnu.org/licenses/>.
+from contextlib import contextmanager
 from enum import Enum
 from typing import Callable
 
 from cubicweb import AuthenticationError, QueryError, Unauthorized, Forbidden
+from cubicweb.pyramid.core import CubicWebPyramidRequest
 from cubicweb.schema_exporters import JSONSchemaExporter
 from rql import RQLException
 from yams import ValidationError, UnknownType
@@ -35,7 +37,7 @@ from cubicweb_api.constants import (
 from cubicweb_api.httperrors import get_http_error, get_http_500_error
 from cubicweb_api.auth.jwt_auth import setup_jwt
 from cubicweb_api.openapi.openapi import setup_openapi
-from cubicweb_api.util import get_cw_repo, get_cw_request, get_transactions_repository
+from cubicweb_api.util import get_cw_repo, get_transactions_repository
 
 log = logging.getLogger(__name__)
 
@@ -79,7 +81,8 @@ def get_route_name(route_name: str) -> str:
     return f"{API_ROUTE_NAME_PREFIX}{route_name}"
 
 
-def catch_rql_errors(func: Callable):
+@contextmanager
+def _catch_rql_errors(cw_req: CubicWebPyramidRequest):
     """
     Calls and returns the result of the given function.
     If an error related to RQL occurs, this will raise an HTTP 400 error
@@ -89,7 +92,11 @@ def catch_rql_errors(func: Callable):
     :raise HTTPError: with 400 code if a RQL related exception is caught
     """
     try:
-        return func()
+        try:
+            yield
+        except ValidationError as e:
+            e.translate(cw_req._)
+            raise
     except (RQLException, QueryError, ValidationError, UnknownType) as e:
         log.info(e.__class__.__name__, exc_info=True)
         raise get_http_error(400, e.__class__.__name__, str(e))
@@ -172,7 +179,8 @@ def rql_route(request: Request):
     request_params = request.openapi_validated.body
     query: str = request_params["query"]
     params: dict = request_params["params"]
-    return catch_rql_errors(lambda: get_cw_request(request).execute(query, params).rows)
+    with _catch_rql_errors(request.cw_cnx):
+        return request.cw_cnx.execute(query, params).rows
 
 
 @view_config(
@@ -218,7 +226,7 @@ def transaction_begin_route(request: Request):
     See the openapi/openapi_template.yml file for more information on this route.
     """
     transactions = get_transactions_repository(request)
-    user = get_cw_request(request).user
+    user = request.cw_cnx.user
     return transactions.begin_transaction(user)
 
 
@@ -237,7 +245,8 @@ def transaction_execute_route(request: Request):
     uuid: str = request_params["uuid"]
     query: str = request_params["query"]
     params: dict = request_params["params"]
-    return catch_rql_errors(lambda: transactions[uuid].execute(query, params).rows)
+    with _catch_rql_errors(request.cw_cnx):
+        return transactions[uuid].execute(query, params).rows
 
 
 @view_config(
@@ -253,7 +262,8 @@ def transaction_commit_route(request: Request):
     transactions = get_transactions_repository(request)
     request_params = request.openapi_validated.body
     uuid: str = request_params["uuid"]
-    return catch_rql_errors(lambda: transactions[uuid].commit())
+    with _catch_rql_errors(request.cw_cnx):
+        return transactions[uuid].commit()
 
 
 @view_config(
